@@ -228,12 +228,6 @@ namespace librbd {
     return 0;
   }
 
-  uint64_t rbd_assign_bid(IoCtx& io_ctx)
-  {
-    Rados rados(io_ctx);
-    return rados.get_instance_id();
-  }
-
   int read_header_bl(IoCtx& io_ctx, const string& header_oid,
 		     bufferlist& header, uint64_t *ver)
   {
@@ -834,9 +828,6 @@ reprotect_and_return_err:
 	     bool old_format, uint64_t features, int *order,
 	     uint64_t stripe_unit, uint64_t stripe_count)
   {
-    if (!order)
-      return -EINVAL;
-
     CephContext *cct = (CephContext *)io_ctx.cct();
     ldout(cct, 20) << "create " << &io_ctx << " name = " << imgname
 		   << " size = " << size << " old_format = " << old_format
@@ -862,6 +853,9 @@ reprotect_and_return_err:
       return -EEXIST;
     }
 
+    if (!order)
+      return -EINVAL;
+
     if (!*order)
       *order = cct->_conf->rbd_default_order;
     if (!*order)
@@ -872,7 +866,8 @@ reprotect_and_return_err:
       return -EDOM;
     }
 
-    uint64_t bid = rbd_assign_bid(io_ctx);
+    Rados rados(io_ctx);
+    uint64_t bid = rados.get_instance_id();
 
     // if striping is enabled, use possibly custom defaults
     if (!old_format && (features & RBD_FEATURE_STRIPINGV2) &&
@@ -1269,19 +1264,6 @@ reprotect_and_return_err:
       return r;
     }
     ictx->parent->snap_set(ictx->parent->snap_name);
-    ictx->parent->parent_lock.get_write();
-    r = refresh_parent(ictx->parent);
-    if (r < 0) {
-      lderr(ictx->cct) << "error refreshing parent snapshot "
-		       << ictx->parent->id << " "
-		       << ictx->parent->snap_name << dendl;
-      ictx->parent->parent_lock.put_write();
-      ictx->parent->snap_lock.put_write();
-      close_image(ictx->parent);
-      ictx->parent = NULL;
-      return r;
-    }
-    ictx->parent->parent_lock.put_write();
     ictx->parent->snap_lock.put_write();
 
     return 0;
@@ -1360,31 +1342,15 @@ reprotect_and_return_err:
     if (r < 0) {
       ldout(cct, 2) << "error opening image: " << cpp_strerror(-r) << dendl;
     } else {
-      string header_oid = ictx->header_oid;
-      old_format = ictx->old_format;
-      unknown_format = false;
-      id = ictx->id;
-
       if (ictx->snaps.size()) {
 	lderr(cct) << "image has snapshots - not removing" << dendl;
 	close_image(ictx);
 	return -ENOTEMPTY;
       }
-
-      std::list<obj_watch_t> watchers;
-      r = io_ctx.list_watchers(header_oid, &watchers);
-      if (r < 0) {
-        lderr(cct) << "error listing watchers" << dendl;
-        close_image(ictx);
-        return r;
-      }
-      if (watchers.size() > 1) {
-        lderr(cct) << "image has watchers - not removing" << dendl;
-        close_image(ictx);
-        return -EBUSY;
-      }
-      assert(watchers.size() == 1);
-
+      string header_oid = ictx->header_oid;
+      old_format = ictx->old_format;
+      unknown_format = false;
+      id = ictx->id;
       ictx->md_lock.get_read();
       trim_image(ictx, 0, prog_ctx);
       ictx->md_lock.put_read();
@@ -1398,7 +1364,6 @@ reprotect_and_return_err:
 				   parent_info.spec, id);
       if (r < 0 && r != -ENOENT) {
 	lderr(cct) << "error removing child from children list" << dendl;
-        close_image(ictx);
 	return r;
       }
       close_image(ictx);
@@ -1511,9 +1476,7 @@ reprotect_and_return_err:
     if (size < ictx->size && ictx->object_cacher) {
       // need to invalidate since we're deleting objects, and
       // ObjectCacher doesn't track non-existent objects
-      r = ictx->invalidate_cache();
-      if (r < 0)
-	return r;
+      ictx->invalidate_cache();
     }
     resize_helper(ictx, size, prog_ctx);
 
@@ -1856,9 +1819,7 @@ reprotect_and_return_err:
     // need to flush any pending writes before resizing and rolling back -
     // writes might create new snapshots. Rolling back will replace
     // the current version, so we have to invalidate that too.
-    r = ictx->invalidate_cache();
-    if (r < 0)
-      return r;
+    ictx->invalidate_cache();
 
     ldout(cct, 2) << "resizing to snapshot size..." << dendl;
     NoOpProgressContext no_op;
@@ -1966,13 +1927,6 @@ reprotect_and_return_err:
 	return;
       }
       assert(m_bl->length() == (size_t)r);
-
-      if (m_bl->is_zero()) {
-	delete m_bl;
-	m_throttle->end_op(r);
-	return;
-      }
-
       Context *ctx = new C_CopyWrite(m_throttle, m_bl);
       AioCompletion *comp = aio_create_completion_internal(ctx, rbd_ctx_cb);
       r = aio_write(m_dest, m_offset, m_bl->length(), m_bl->c_str(), comp);
@@ -2082,7 +2036,7 @@ reprotect_and_return_err:
 			 << ictx->snap_name << "'" << dendl;
     int r = ictx->init();
     if (r < 0)
-      goto err_close;
+      return r;
 
     if (!ictx->read_only) {
       r = ictx->register_watch();
@@ -2886,19 +2840,6 @@ reprotect_and_return_err:
     return r;
   }
 
-  int invalidate_cache(ImageCtx *ictx)
-  {
-    CephContext *cct = ictx->cct;
-    ldout(cct, 20) << "invalidate_cache " << ictx << dendl;
-
-    int r = ictx_check(ictx);
-    if (r < 0)
-      return r;
-
-    RWLock::WLocker l(ictx->md_lock);
-    return ictx->invalidate_cache();
-  }
-
   int aio_write(ImageCtx *ictx, uint64_t off, size_t len, const char *buf,
 		AioCompletion *c)
   {
