@@ -147,84 +147,106 @@ void PGMonitor::tick()
 void PGMonitor::create_initial()
 {
   dout(10) << "create_initial -- creating initial map" << dendl;
+  format_version = 1;
 }
 
 void PGMonitor::update_from_paxos(bool *need_bootstrap)
 {
-  version_t version = get_version();
+  version_t version = get_last_committed();
   if (version == pg_map.version)
     return;
   assert(version >= pg_map.version);
 
-  /* Obtain latest full pgmap version, if available and whose version is
-   * greater than the current pgmap's version.
-   */
-  version_t latest_full = get_version_latest_full();
-  if ((latest_full > 0) && (latest_full > pg_map.version)) {
-    bufferlist latest_bl;
-    int err = get_version_full(latest_full, latest_bl);
-    assert(err == 0);
-    dout(7) << __func__ << " loading latest full pgmap v"
-	    << latest_full << dendl;
-    try {
-      PGMap tmp_pg_map;
-      bufferlist::iterator p = latest_bl.begin();
-      tmp_pg_map.decode(p);
-      pg_map = tmp_pg_map;
-    } catch (const std::exception& e) {
-      dout(0) << __func__ << ": error parsing update: "
-	      << e.what() << dendl;
-      assert(0 == "update_from_paxos: error parsing update");
-      return;
-    }
-  }
+  if (format_version == 0) {
+    // old format
 
-  // walk through incrementals
-  while (version > pg_map.version) {
-    bufferlist bl;
-    int err = get_version(pg_map.version+1, bl);
-    assert(err == 0);
-    assert(bl.length());
-
-    dout(7) << "update_from_paxos  applying incremental " << pg_map.version+1 << dendl;
-    PGMap::Incremental inc;
-    try {
-      bufferlist::iterator p = bl.begin();
-      inc.decode(p);
-    }
-    catch (const std::exception &e) {
-      dout(0) << "update_from_paxos: error parsing "
-	      << "incremental update: " << e.what() << dendl;
-      assert(0 == "update_from_paxos: error parsing incremental update");
-      return;
+    /* Obtain latest full pgmap version, if available and whose version is
+     * greater than the current pgmap's version.
+     */
+    version_t latest_full = get_version_latest_full();
+    if ((latest_full > 0) && (latest_full > pg_map.version)) {
+      bufferlist latest_bl;
+      int err = get_version_full(latest_full, latest_bl);
+      assert(err == 0);
+      dout(7) << __func__ << " loading latest full pgmap v"
+	      << latest_full << dendl;
+      try {
+	PGMap tmp_pg_map;
+	bufferlist::iterator p = latest_bl.begin();
+	tmp_pg_map.decode(p);
+	pg_map = tmp_pg_map;
+      } catch (const std::exception& e) {
+	dout(0) << __func__ << ": error parsing update: "
+		<< e.what() << dendl;
+	assert(0 == "update_from_paxos: error parsing update");
+	return;
+      }
     }
 
-    pg_map.apply_incremental(g_ceph_context, inc);
-    
-    dout(10) << pg_map << dendl;
+    // walk through incrementals
+    while (version > pg_map.version) {
+      bufferlist bl;
+      int err = get_version(pg_map.version+1, bl);
+      assert(err == 0);
+      assert(bl.length());
 
-    if (inc.pg_scan)
+      dout(7) << "update_from_paxos  applying incremental " << pg_map.version+1 << dendl;
+      PGMap::Incremental inc;
+      try {
+	bufferlist::iterator p = bl.begin();
+	inc.decode(p);
+      }
+      catch (const std::exception &e) {
+	dout(0) << "update_from_paxos: error parsing "
+		<< "incremental update: " << e.what() << dendl;
+	assert(0 == "update_from_paxos: error parsing incremental update");
+	return;
+      }
+
+      pg_map.apply_incremental(g_ceph_context, inc);
+
+      dout(10) << pg_map << dendl;
+
+      if (inc.pg_scan)
+	last_sent_pg_create.clear();  // reset pg_create throttle timer
+    }
+
+  } else if (format_version == 1) {
+    // pg/osd keys in leveldb
+
+    // read meta
+    epoch_t last_pg_scan = pg_map.last_pg_scan;
+
+    while (version > pg_map.version) {
+      // load full state?
+      if (pg_map.version == 0) {
+	dout(10) << __func__ << " v0, read_full" << dendl;
+	read_pgmap_full();
+	goto out;
+      }
+
+      // incremental state?
+      dout(10) << __func__ << " read_incremental" << dendl;
+      bufferlist bl;
+      int r = get_version(pg_map.version + 1, bl);
+      if (r == -ENOENT) {
+	dout(10) << __func__ << " failed to read_incremental, read_full" << dendl;
+	read_pgmap_full();
+	goto out;
+      }
+      assert(r == 0);
+      apply_pgmap_delta(bl);
+    }
+
+    read_pgmap_meta();
+
+  out:
+    if (last_pg_scan != pg_map.last_pg_scan)
       last_sent_pg_create.clear();  // reset pg_create throttle timer
   }
 
   assert(version == pg_map.version);
 
-  /* If we dump the summaries onto the k/v store, they hardly would be useful
-   * without a tool created with reading them in mind.
-   * Comment this out until we decide what is the best course of action.
-   *
-  // dump pgmap summaries?  (useful for debugging)
-  if (0) {
-    stringstream ds;
-    pg_map.dump(ds);
-    bufferlist d;
-    d.append(ds);
-    mon->store->put_bl_sn(d, "pgmap_dump", version);
-  }
-  */
-
-  update_trim();
-
   if (mon->osdmon()->osdmap.get_epoch()) {
     map_pg_creates();
     send_pg_creates();
@@ -233,7 +255,29 @@ void PGMonitor::update_from_paxos(bool *need_bootstrap)
   update_logger();
 }
 
-void PGMonitor::init()
+void PGMonitor::on_upgrade()
+{
+  dout(1) << __func__ << " discarding in-core PGMap" << dendl;
+  pg_map = PGMap();
+}
+
+void PGMonitor::upgrade_format()
+{
+  unsigned current = 1;
+  assert(format_version <= current);
+  if (format_version == current)
+    return;
+
+  dout(1) << __func__ << " to " << current << dendl;
+
+  // upgrade by dirtying it all
+  pg_map.dirty_all(pending_inc);
+
+  format_version = current;
+  propose_pending();
+}
+
+void PGMonitor::post_paxos_update()
 {
   if (mon->osdmon()->osdmap.get_epoch()) {
     map_pg_creates();
@@ -274,41 +318,211 @@ void PGMonitor::create_pending()
   dout(10) << "create_pending v " << pending_inc.version << dendl;
 }
 
+void PGMonitor::read_pgmap_meta()
+{
+  dout(10) << __func__ << dendl;
+
+  string prefix = pgmap_meta_prefix;
+
+  version_t version = mon->store->get(prefix, "version");
+  epoch_t last_osdmap_epoch = mon->store->get(prefix, "last_osdmap_epoch");
+  epoch_t last_pg_scan = mon->store->get(prefix, "last_pg_scan");
+  pg_map.set_version(version);
+  pg_map.set_last_osdmap_epoch(last_osdmap_epoch);
+
+  if (last_pg_scan != pg_map.get_last_pg_scan()) {
+    pg_map.set_last_pg_scan(last_pg_scan);
+    // clear our osdmap epoch so that map_pg_creates() will re-run
+    last_map_pg_create_osd_epoch = 0;
+  }
+
+  float full_ratio, nearfull_ratio;
+  {
+    bufferlist bl;
+    mon->store->get(prefix, "full_ratio", bl);
+    bufferlist::iterator p = bl.begin();
+    ::decode(full_ratio, p);
+  }
+  {
+    bufferlist bl;
+    mon->store->get(prefix, "nearfull_ratio", bl);
+    bufferlist::iterator p = bl.begin();
+    ::decode(nearfull_ratio, p);
+  }
+  pg_map.set_full_ratios(full_ratio, nearfull_ratio);
+  {
+    bufferlist bl;
+    mon->store->get(prefix, "stamp", bl);
+    bufferlist::iterator p = bl.begin();
+    utime_t stamp;
+    ::decode(stamp, p);
+    pg_map.set_stamp(stamp);
+  }
+}
+
+void PGMonitor::read_pgmap_full()
+{
+  read_pgmap_meta();
+
+  string prefix = pgmap_pg_prefix;
+  for (KeyValueDB::Iterator i = mon->store->get_iterator(prefix); i->valid(); i->next()) {
+    string key = i->key();
+    pg_t pgid;
+    if (!pgid.parse(key.c_str())) {
+      dout(0) << "unable to parse key " << key << dendl;
+      continue;
+    }
+    bufferlist bl = i->value();
+    pg_map.update_pg(pgid, bl);
+    dout(20) << " got " << pgid << dendl;
+  }
+
+  prefix = pgmap_osd_prefix;
+  for (KeyValueDB::Iterator i = mon->store->get_iterator(prefix); i->valid(); i->next()) {
+    string key = i->key();
+    int osd = atoi(key.c_str());
+    bufferlist bl = i->value();
+    pg_map.update_osd(osd, bl);
+    dout(20) << " got osd." << osd << dendl;
+  }
+}
+
+void PGMonitor::apply_pgmap_delta(bufferlist& bl)
+{
+  version_t v = pg_map.version + 1;
+
+  utime_t inc_stamp;
+  bufferlist dirty_pgs, dirty_osds;
+  {
+    bufferlist::iterator p = bl.begin();
+    ::decode(inc_stamp, p);
+    ::decode(dirty_pgs, p);
+    ::decode(dirty_osds, p);
+  }
+
+  pool_stat_t pg_sum_old = pg_map.pg_sum;
+
+  // pgs
+  bufferlist::iterator p = dirty_pgs.begin();
+  while (!p.end()) {
+    pg_t pgid;
+    ::decode(pgid, p);
+    dout(20) << " refreshing pg " << pgid << dendl;
+    bufferlist bl;
+    int r = mon->store->get(pgmap_pg_prefix, stringify(pgid), bl);
+    if (r >= 0) {
+      pg_map.update_pg(pgid, bl);
+    } else {
+      pg_map.remove_pg(pgid);
+    }
+  }
+
+  // osds
+  p = dirty_osds.begin();
+  while (!p.end()) {
+    int32_t osd;
+    ::decode(osd, p);
+    dout(20) << " refreshing osd." << osd << dendl;
+    bufferlist bl;
+    int r = mon->store->get(pgmap_osd_prefix, stringify(osd), bl);
+    if (r >= 0) {
+      pg_map.update_osd(osd, bl);
+    } else {
+      pg_map.remove_osd(osd);
+    }
+  }
+
+  pg_map.update_delta(g_ceph_context, inc_stamp, pg_sum_old);
+
+  // ok, we're now on the new version
+  pg_map.version = v;
+}
+
+
 void PGMonitor::encode_pending(MonitorDBStore::Transaction *t)
 {
   version_t version = pending_inc.version;
   dout(10) << __func__ << " v " << version << dendl;
-  assert(get_version() + 1 == version);
+  assert(get_last_committed() + 1 == version);
   pending_inc.stamp = ceph_clock_now(g_ceph_context);
 
-  bufferlist bl;
-  pending_inc.encode(bl, mon->get_quorum_features());
+  uint64_t features = mon->get_quorum_features();
+
+  string prefix = pgmap_meta_prefix;
+
+  t->put(prefix, "version", pending_inc.version);
+  {
+    bufferlist bl;
+    ::encode(pending_inc.stamp, bl);
+    t->put(prefix, "stamp", bl);
+  }
+
+  if (pending_inc.osdmap_epoch)
+    t->put(prefix, "last_osdmap_epoch", pending_inc.osdmap_epoch);
+  if (pending_inc.pg_scan)
+    t->put(prefix, "last_pg_scan", pending_inc.pg_scan);
+  if (pending_inc.full_ratio > 0) {
+    bufferlist bl;
+    ::encode(pending_inc.full_ratio, bl);
+    t->put(prefix, "full_ratio", bl);
+  }
+  if (pending_inc.nearfull_ratio > 0) {
+    bufferlist bl;
+    ::encode(pending_inc.nearfull_ratio, bl);
+    t->put(prefix, "nearfull_ratio", bl);
+  }
+
+  bufferlist incbl;
+  ::encode(pending_inc.stamp, incbl);
+  {
+    bufferlist dirty;
+    string prefix = pgmap_pg_prefix;
+    for (map<pg_t,pg_stat_t>::const_iterator p = pending_inc.pg_stat_updates.begin();
+	 p != pending_inc.pg_stat_updates.end();
+	 ++p) {
+      ::encode(p->first, dirty);
+      bufferlist bl;
+      ::encode(p->second, bl, features);
+      t->put(prefix, stringify(p->first), bl);
+    }
+    for (set<pg_t>::const_iterator p = pending_inc.pg_remove.begin(); p != pending_inc.pg_remove.end(); ++p) {
+      ::encode(*p, dirty);
+      t->erase(prefix, stringify(*p));
+    }
+    ::encode(dirty, incbl);
+  }
+  {
+    bufferlist dirty;
+    string prefix = pgmap_osd_prefix;
+    for (map<int32_t,osd_stat_t>::const_iterator p = pending_inc.osd_stat_updates.begin();
+	 p != pending_inc.osd_stat_updates.end();
+	 ++p) {
+      ::encode(p->first, dirty);
+      bufferlist bl;
+      ::encode(p->second, bl, features);
+      t->put(prefix, stringify(p->first), bl);
+    }
+    for (set<int32_t>::const_iterator p = pending_inc.osd_stat_rm.begin(); p != pending_inc.osd_stat_rm.end(); ++p) {
+      ::encode(*p, dirty);
+      t->erase(prefix, stringify(*p));
+    }
+    ::encode(dirty, incbl);
+  }
+
+  put_version(t, version, incbl);
 
-  put_version(t, version, bl);
   put_last_committed(t, version);
 }
 
-void PGMonitor::encode_full(MonitorDBStore::Transaction *t)
-{
-  dout(10) << __func__ << " pgmap v " << pg_map.version << dendl;
-  assert(get_version() == pg_map.version);
-
-  bufferlist full_bl;
-  pg_map.encode(full_bl, mon->get_quorum_features());
-
-  put_version_full(t, pg_map.version, full_bl);
-  put_version_latest_full(t, pg_map.version);
-}
-
-void PGMonitor::update_trim()
+version_t PGMonitor::get_trim_to()
 {
   unsigned max = g_conf->mon_max_pgmap_epochs;
-  version_t version = get_version();
+  version_t version = get_last_committed();
   if (mon->is_leader() && (version > max))
-    set_trim_to(version - max);
+    return version - max;
+  return 0;
 }
 
-
 bool PGMonitor::preprocess_query(PaxosServiceMessage *m)
 {
   dout(10) << "preprocess_query " << *m << " from " << m->get_orig_source_inst() << dendl;
@@ -371,7 +585,7 @@ void PGMonitor::handle_statfs(MStatfs *statfs)
   }
 
   // fill out stfs
-  reply = new MStatfsReply(mon->monmap->fsid, statfs->get_tid(), get_version());
+  reply = new MStatfsReply(mon->monmap->fsid, statfs->get_tid(), get_last_committed());
 
   // these are in KB.
   reply->h.st.kb = pg_map.osd_sum.kb;
@@ -403,7 +617,7 @@ bool PGMonitor::preprocess_getpoolstats(MGetPoolStats *m)
     goto out;
   }
   
-  reply = new MGetPoolStatsReply(m->fsid, m->get_tid(), get_version());
+  reply = new MGetPoolStatsReply(m->fsid, m->get_tid(), get_last_committed());
 
   for (list<string>::iterator p = m->pools.begin();
        p != m->pools.end();
@@ -468,7 +682,8 @@ bool PGMonitor::pg_stats_have_changed(int from, const MPGStats *stats) const
     hash_map<pg_t,pg_stat_t>::const_iterator t = pg_map.pg_stat.find(p->first);
     if (t == pg_map.pg_stat.end())
       return true;
-    if (t->second.reported != p->second.reported)
+    if (t->second.reported_epoch != p->second.reported_epoch ||
+	t->second.reported_seq != p->second.reported_seq)
       return true;
   }
 
@@ -502,7 +717,7 @@ bool PGMonitor::prepare_pg_stats(MPGStats *stats)
     for (map<pg_t,pg_stat_t>::const_iterator p = stats->pg_stat.begin();
 	 p != stats->pg_stat.end();
 	 ++p) {
-      ack->pg_stat[p->first] = p->second.reported;
+      ack->pg_stat[p->first] = make_pair(p->second.reported_seq, p->second.reported_epoch);
     }
     mon->send_reply(stats, ack);
     stats->put();
@@ -517,18 +732,6 @@ bool PGMonitor::prepare_pg_stats(MPGStats *stats)
   else
     dout(10) << " got osd." << from << " " << stats->osd_stat << " (first report)" << dendl;
 
-  // apply to live map too (screw consistency)
-  /*
-    actually, no, don't.  that screws up our "latest" stash.  and can
-    lead to weird output where things appear to jump backwards in
-    time... that's just confusing.
-
-  if (pg_map.osd_stat.count(from))
-    pg_map.stat_osd_sub(pg_map.osd_stat[from]);
-  pg_map.osd_stat[from] = stats->osd_stat;
-  pg_map.stat_osd_add(stats->osd_stat);
-  */
-
   // pg stats
   MPGStatsAck *ack = new MPGStatsAck;
   ack->set_tid(stats->get_tid());
@@ -536,22 +739,24 @@ bool PGMonitor::prepare_pg_stats(MPGStats *stats)
        p != stats->pg_stat.end();
        ++p) {
     pg_t pgid = p->first;
-    ack->pg_stat[pgid] = p->second.reported;
+    ack->pg_stat[pgid] = make_pair(p->second.reported_seq, p->second.reported_epoch);
 
-    if ((pg_map.pg_stat.count(pgid) && 
-	 pg_map.pg_stat[pgid].reported > p->second.reported)) {
-      dout(15) << " had " << pgid << " from " << pg_map.pg_stat[pgid].reported << dendl;
+    if (pg_map.pg_stat.count(pgid) &&
+        pg_map.pg_stat[pgid].get_version_pair() > p->second.get_version_pair()) {
+      dout(15) << " had " << pgid << " from " << pg_map.pg_stat[pgid].reported_epoch << ":"
+	       << pg_map.pg_stat[pgid].reported_seq << dendl;
       continue;
     }
     if (pending_inc.pg_stat_updates.count(pgid) && 
-	pending_inc.pg_stat_updates[pgid].reported > p->second.reported) {
-      dout(15) << " had " << pgid << " from " << pending_inc.pg_stat_updates[pgid].reported
-	       << " (pending)" << dendl;
+        pending_inc.pg_stat_updates[pgid].get_version_pair() > p->second.get_version_pair()) {
+      dout(15) << " had " << pgid << " from " << pending_inc.pg_stat_updates[pgid].reported_epoch << ":"
+	       << pending_inc.pg_stat_updates[pgid].reported_seq << " (pending)" << dendl;
       continue;
     }
 
     if (pg_map.pg_stat.count(pgid) == 0) {
-      dout(15) << " got " << pgid << " reported at " << p->second.reported
+      dout(15) << " got " << pgid << " reported at " << p->second.reported_epoch << ":"
+	       << p->second.reported_seq
 	       << " state " << pg_state_string(p->second.state)
 	       << " but DNE in pg_map; pool was probably deleted."
 	       << dendl;
@@ -559,7 +764,7 @@ bool PGMonitor::prepare_pg_stats(MPGStats *stats)
     }
       
     dout(15) << " got " << pgid
-	     << " reported at " << p->second.reported
+	     << " reported at " << p->second.reported_epoch << ":" << p->second.reported_seq
 	     << " state " << pg_state_string(pg_map.pg_stat[pgid].state)
 	     << " -> " << pg_state_string(p->second.state)
 	     << dendl;
@@ -813,7 +1018,14 @@ bool PGMonitor::register_new_pgs()
 
 void PGMonitor::map_pg_creates()
 {
-  dout(10) << "map_pg_creates to " << pg_map.creating_pgs.size() << " pgs" << dendl;
+  OSDMap *osdmap = &mon->osdmon()->osdmap;
+  if (osdmap->get_epoch() == last_map_pg_create_osd_epoch) {
+    dout(10) << "map_pg_creates to " << pg_map.creating_pgs.size() << " pgs -- no change" << dendl;
+    return;
+  }
+
+  dout(10) << "map_pg_creates to " << pg_map.creating_pgs.size() << " pgs osdmap epoch " << osdmap->get_epoch() << dendl;
+  last_map_pg_create_osd_epoch = osdmap->get_epoch();
 
   for (set<pg_t>::iterator p = pg_map.creating_pgs.begin();
        p != pg_map.creating_pgs.end();
@@ -824,7 +1036,7 @@ void PGMonitor::map_pg_creates()
     if (s.parent_split_bits)
       on = s.parent;
     vector<int> acting;
-    int nrep = mon->osdmon()->osdmap.pg_to_acting_osds(on, acting);
+    int nrep = osdmap->pg_to_acting_osds(on, acting);
 
     if (s.acting.size()) {
       pg_map.creating_pgs_by_osd[s.acting[0]].erase(pgid);
@@ -1089,6 +1301,8 @@ void PGMonitor::dump_info(Formatter *f)
   f->open_object_section("pgmap");
   pg_map.dump(f);
   f->close_section();
+
+  f->dump_unsigned("pgmap_first_committed", get_first_committed());
 }
 
 bool PGMonitor::preprocess_command(MMonCommand *m)
@@ -1101,7 +1315,7 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
   if (!cmdmap_from_json(m->cmd, &cmdmap, ss)) {
     // ss has reason for failure
     string rs = ss.str();
-    mon->reply_command(m, -EINVAL, rs, rdata, get_version());
+    mon->reply_command(m, -EINVAL, rs, rdata, get_last_committed());
     return true;
   }
 
@@ -1109,21 +1323,23 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
   cmd_getval(g_ceph_context, cmdmap, "prefix", prefix);
 
   MonSession *session = m->get_session();
-  if (!session ||
-      (!session->is_capable("pg", MON_CAP_R) &&
-       !mon->_allowed_command(session, cmdmap))) {
-    mon->reply_command(m, -EACCES, "access denied", rdata, get_version());
+  if (!session) {
+    mon->reply_command(m, -EACCES, "access denied", rdata, get_last_committed());
     return true;
   }
 
   // perhaps these would be better in the parsing, but it's weird
   if (prefix == "pg dump_json") {
+    vector<string> v;
+    v.push_back(string("all"));
     cmd_putval(g_ceph_context, cmdmap, "format", string("json"));
-    cmd_putval(g_ceph_context, cmdmap, "dumpcontents", string("all"));
+    cmd_putval(g_ceph_context, cmdmap, "dumpcontents", v);
     prefix = "pg dump";
   } else if (prefix == "pg dump_pools_json") {
+    vector<string> v;
+    v.push_back(string("pools"));
     cmd_putval(g_ceph_context, cmdmap, "format", string("json"));
-    cmd_putval(g_ceph_context, cmdmap, "dumpcontents", string("pool"));
+    cmd_putval(g_ceph_context, cmdmap, "dumpcontents", v);
     prefix = "pg dump";
   }
 
@@ -1132,8 +1348,14 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
   boost::scoped_ptr<Formatter> f(new_formatter(format));
 
   if (prefix == "pg stat") {
-    ds << pg_map;
-    rdata.append(ds);
+    if (f) {
+      f->open_object_section("pg_map");
+      pg_map.dump(f.get());
+      f->close_section();
+      f->flush(ds);
+    } else {
+      ds << pg_map;
+    }
     r = 0;
   } else if (prefix == "pg getmap") {
     pg_map.encode(rdata);
@@ -1179,7 +1401,10 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
 	  pg_map.dump_osd_stats(f.get());
 	}
 	if (what.count("pgs")) {
-	  pg_map.dump_pg_stats(f.get());
+	  pg_map.dump_pg_stats(f.get(), false);
+	}
+	if (what.count("pgs_brief")) {
+	  pg_map.dump_pg_stats(f.get(), true);
 	}
       }
       f->flush(ds);
@@ -1198,7 +1423,6 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
     cmd_getval(g_ceph_context, cmdmap, "threshold", threshold,
 	       int64_t(g_conf->mon_pg_stuck_threshold));
 
-    boost::scoped_ptr<Formatter> f(new_formatter("json"));
     r = dump_stuck_pg_stats(ds, f.get(), (int)threshold, stuckop_vec);
     ss << "ok";
     r = 0;
@@ -1219,9 +1443,29 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
     }
     pg_t mpgid = mon->osdmon()->osdmap.raw_pg_to_pg(pgid);
     mon->osdmon()->osdmap.pg_to_up_acting_osds(pgid, up, acting);
-    ds << "osdmap e" << mon->osdmon()->osdmap.get_epoch()
-       << " pg " << pgid << " (" << mpgid << ")"
-       << " -> up " << up << " acting " << acting;
+    if (f) {
+      f->open_object_section("pg_map");
+      f->dump_stream("epoch") << mon->osdmon()->osdmap.get_epoch();
+      f->dump_stream("raw_pgid") << pgid;
+      f->dump_stream("pgid") << mpgid;
+
+      f->open_array_section("up");
+      for (vector<int>::iterator it = up.begin(); it != up.end(); ++it)
+	f->dump_int("up_osd", *it);
+      f->close_section();
+
+      f->open_array_section("acting");
+      for (vector<int>::iterator it = acting.begin(); it != acting.end(); ++it)
+	f->dump_int("acting_osd", *it);
+      f->close_section();
+
+      f->close_section();
+      f->flush(ds);
+    } else {
+      ds << "osdmap e" << mon->osdmon()->osdmap.get_epoch()
+	 << " pg " << pgid << " (" << mpgid << ")"
+	 << " -> up " << up << " acting " << acting;
+    }
     r = 0;
   } else if (prefix == "pg scrub" || 
 	     prefix == "pg repair" || 
@@ -1302,7 +1546,7 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
   string rs;
   getline(ss, rs);
   rdata.append(ds);
-  mon->reply_command(m, r, rs, rdata, get_version());
+  mon->reply_command(m, r, rs, rdata, get_last_committed());
   return true;
 }
 
@@ -1318,7 +1562,7 @@ bool PGMonitor::prepare_command(MMonCommand *m)
   if (!cmdmap_from_json(m->cmd, &cmdmap, ss)) {
     // ss has reason for failure
     string rs = ss.str();
-    mon->reply_command(m, -EINVAL, rs, get_version());
+    mon->reply_command(m, -EINVAL, rs, get_last_committed());
     return true;
   }
 
@@ -1326,10 +1570,8 @@ bool PGMonitor::prepare_command(MMonCommand *m)
   cmd_getval(g_ceph_context, cmdmap, "prefix", prefix);
 
   MonSession *session = m->get_session();
-  if (!session ||
-      (!session->is_capable("pg", MON_CAP_W) &&
-       !mon->_allowed_command(session, cmdmap))) {
-    mon->reply_command(m, -EACCES, "access denied", get_version());
+  if (!session) {
+    mon->reply_command(m, -EACCES, "access denied", get_last_committed());
     return true;
   }
 
@@ -1378,12 +1620,12 @@ bool PGMonitor::prepare_command(MMonCommand *m)
   getline(ss, rs);
   if (r < 0 && rs.length() == 0)
     rs = cpp_strerror(r);
-  mon->reply_command(m, r, rs, get_version());
+  mon->reply_command(m, r, rs, get_last_committed());
   return false;
 
  update:
   getline(ss, rs);
-  wait_for_finished_proposal(new Monitor::C_Command(mon, m, r, rs, get_version()));
+  wait_for_finished_proposal(new Monitor::C_Command(mon, m, r, rs, get_last_committed()));
   return true;
 }
 
@@ -1426,6 +1668,26 @@ static void note_stuck_detail(enum PGMap::StuckPG what,
   }
 }
 
+int PGMonitor::_warn_slow_request_histogram(const pow2_hist_t& h, string suffix,
+					    list<pair<health_status_t,string> >& summary,
+					    list<pair<health_status_t,string> > *detail) const
+{
+  unsigned sum = 0;
+  for (unsigned i = h.h.size() - 1; i > 0; --i) {
+    float ub = (float)(1 << i) / 1000.0;
+    if (ub < g_conf->mon_osd_max_op_age)
+      break;
+    ostringstream ss;
+    if (h.h[i]) {
+      ss << h.h[i] << " ops are blocked > " << ub << " sec" << suffix;
+      if (detail)
+	detail->push_back(make_pair(HEALTH_WARN, ss.str()));
+      sum += h.h[i];
+    }
+  }
+  return sum;
+}
+
 void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
 			   list<pair<health_status_t,string> > *detail) const
 {
@@ -1530,17 +1792,48 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
     }
   }
 
+  // slow requests
+  if (g_conf->mon_osd_max_op_age > 0 &&
+      pg_map.osd_sum.op_queue_age_hist.upper_bound() > g_conf->mon_osd_max_op_age) {
+    unsigned sum = _warn_slow_request_histogram(pg_map.osd_sum.op_queue_age_hist, "", summary, detail);
+    if (sum > 0) {
+      ostringstream ss;
+      ss << sum << " requests are blocked > " << g_conf->mon_osd_max_op_age << " sec";
+      summary.push_back(make_pair(HEALTH_WARN, ss.str()));
+
+      if (detail) {
+        unsigned num_slow_osds = 0;
+	// do per-osd warnings
+	for (hash_map<int32_t,osd_stat_t>::const_iterator p = pg_map.osd_stat.begin();
+	     p != pg_map.osd_stat.end();
+	     ++p) {
+	  if (_warn_slow_request_histogram(p->second.op_queue_age_hist,
+					   string(" on osd.") + stringify(p->first),
+					   summary, detail))
+	    ++num_slow_osds;
+	}
+	ostringstream ss2;
+	ss2 << num_slow_osds << " osds have slow requests";
+	summary.push_back(make_pair(HEALTH_WARN, ss2.str()));
+	detail->push_back(make_pair(HEALTH_WARN, ss2.str()));
+      }
+    }
+  }
+
+  // recovery
   stringstream rss;
-  pg_map.recovery_summary(rss);
+  pg_map.recovery_summary(NULL, &rss);
   if (!rss.str().empty()) {
     summary.push_back(make_pair(HEALTH_WARN, "recovery " + rss.str()));
     if (detail)
       detail->push_back(make_pair(HEALTH_WARN, "recovery " + rss.str()));
   }
   
+  // full/nearfull
   check_full_osd_health(summary, detail, pg_map.full_osds, "full", HEALTH_ERR);
   check_full_osd_health(summary, detail, pg_map.nearfull_osds, "near full", HEALTH_WARN);
 
+  // scrub
   if (pg_map.pg_sum.stats.sum.num_scrub_errors) {
     ostringstream ss;
     ss << pg_map.pg_sum.stats.sum.num_scrub_errors << " scrub errors";
