@@ -15,6 +15,12 @@ static const char *signed_subresources[] = {
   "partNumber",
   "policy",
   "requestPayment",
+  "response-cache-control",
+  "response-content-disposition",
+  "response-content-encoding",
+  "response-content-language",
+  "response-content-type",
+  "response-expires",
   "torrent",
   "uploadId",
   "uploads",
