@@ -758,7 +758,7 @@ struct RGWBucketStats
 
 struct req_state;
 
-struct RGWEnv;
+class RGWEnv;
 
 class RGWClientIO;
 
@@ -801,13 +801,12 @@ struct req_state {
    uint32_t perm_mask;
    utime_t header_time;
 
+   const char *bucket_name;
    const char *object;
 
    rgw_bucket bucket;
    string bucket_name_str;
    string object_str;
-   string src_bucket_name;
-   string src_object;
    ACLOwner bucket_owner;
    ACLOwner owner;
 
@@ -822,6 +821,7 @@ struct req_state {
    RGWUserInfo user; 
    RGWAccessControlPolicy *bucket_acl;
    RGWAccessControlPolicy *object_acl;
+   RGWCORSConfiguration   *bucket_cors;
 
    bool system_request;
 
@@ -847,7 +847,7 @@ struct req_state {
 
    req_info info;
 
-   req_state(CephContext *_cct, struct RGWEnv *e);
+   req_state(CephContext *_cct, class RGWEnv *e);
    ~req_state();
 };
 
@@ -1234,7 +1234,6 @@ extern bool verify_object_permission(struct req_state *s, int perm);
 /** Convert an input URL into a sane object name
  * by converting %-escaped strings into characters, etc*/
 extern bool url_decode(string& src_str, string& dest_str);
-extern void url_encode(const string& src, string& dst);
 
 extern void calc_hmac_sha1(const char *key, int key_len,
                           const char *msg, int msg_len, char *dest);
