@@ -34,10 +34,8 @@ protected:
   struct req_state *s;
   RGWHandler *dialect_handler;
   RGWRados *store;
-  RGWCORSConfiguration bucket_cors;
-  bool cors_exist;
 public:
-  RGWOp() : s(NULL), dialect_handler(NULL), store(NULL), cors_exist(false) {}
+  RGWOp() : s(NULL), dialect_handler(NULL), store(NULL) {}
   virtual ~RGWOp() {}
 
   virtual void init(RGWRados *store, struct req_state *s, RGWHandler *dialect_handler) {
@@ -45,9 +43,6 @@ public:
     this->s = s;
     this->dialect_handler = dialect_handler;
   }
-  int read_bucket_cors();
-  bool generate_cors_headers(string& origin, string& method, string& headers, string& exp_headers, unsigned *max_age);
-
   virtual int verify_params() { return 0; }
   virtual bool prefetch_data() { return false; }
   virtual int verify_permission() = 0;
@@ -531,6 +526,7 @@ public:
 class RGWGetCORS : public RGWOp {
 protected:
   int ret;
+  RGWCORSConfiguration bucket_cors;
 
 public:
   RGWGetCORS() : ret(0) {}
@@ -590,7 +586,7 @@ public:
   }
 
   int verify_permission() {return 0;}
-  int validate_cors_request(RGWCORSConfiguration *cc);
+  int validate_cors_request();
   void execute();
   void get_response_params(string& allowed_hdrs, string& exp_hdrs, unsigned *max_age);
   virtual void send_response() = 0;
